local LEVELS = {
	-- Level 1.
	{
		playerStartX = 35,
		playerStartY = 300,

		platforms = {
			{x=0,    y=400, w=300, h=100}, -- Spawn point.
			{x=400,  y=280, w=300, h=100},
			{x=400,  y=80,  w=300, h=100},
			{x=800,  y=180, w=100, h=25},
			{x=800,  y=1,   w=300, h=25},
			{x=1400, y=100, w=300, h=100},
			{x=1900, y=100, w=25,  h=25},
			{x=2100, y=100, w=25,  h=25},
			{x=2300, y=100, w=25,  h=25},
			{x=2500, y=100, w=25,  h=25},
			{x=2800, y=280, w=300, h=100},
		},

		dangerZones = {
			{x=0, y=550, w=10000000, h=50},
		},

		endRectangle = {x=2800, y=280, w=300, h=100},
	},
	-- Level 2.
	{
		playerStartX = 35,
		playerStartY = 300,

		platforms = {
			{x=0, y=400, w=300, h=100}, -- Spawn point.
		},

		dangerZones = {
			{x=0, y=550, w=10000000, h=50},
		},

		endRectangle = {x=500, y=400, w=300, h=100},
	},
	-- and so on...
}

local function startLevel(levelData)
	print("Starting new level")

	local cameraModule = require 'hump.camera'

	-- Whenver we set a global variable it's good to use _G. as prefix.
	-- It makes the code, and our intention, clearer.
	_G.cam = cameraModule() -- `_G.cam` and `cam` effectively means the same thing.

	local wf = require 'windfield.windfield'
	_G.world = wf.newWorld(0, 900, false)
	world:setQueryDebugDrawing(true)

	world:addCollisionClass('platform')
	world:addCollisionClass('player'--[[, {ignores = {'platform'}}]])
	world:addCollisionClass('danger')
	world:addCollisionClass('end')

	-- Spawn all things in the world.

	_G.player    = world:newRectangleCollider(levelData.playerStartX, levelData.playerStartY, 80, 80, {collision_class = "player"})
	player.speed = 240
	player:setFixedRotation(true)

	for _, platformData in ipairs(levelData.platforms) do
		local platform = world:newRectangleCollider(platformData.x, platformData.y, platformData.w, platformData.h, {collision_class = "platform"})
		platform:setType('static')
	end

	for _, dangerZoneData in ipairs(levelData.dangerZones) do
		local dangerZone = world:newRectangleCollider(dangerZoneData.x, dangerZoneData.y, dangerZoneData.w, dangerZoneData.h, {collision_class = "danger"})
		dangerZone:setType('static')
	end

	local platform = world:newRectangleCollider(levelData.endRectangle.x, levelData.endRectangle.y, levelData.endRectangle.w, levelData.endRectangle.h, {collision_class = "end"})
	platform:setType('static')
end

function love.load()
	love.window.setMode(1000, 768)

	_G.currentLevelIndex = 1

	startLevel(LEVELS[currentLevelIndex])
end

function love.keypressed(key)
	if key == 'space' then
		local colliders = world:queryRectangleArea(player:getX() - 40, player:getY() + 40, 80, 1, {'platform'})
		if #colliders > 0 then
			player:applyLinearImpulse(0, -6001)
		end
	end
end

function love.update(dt)
	if world then
		-- First, update the player's position according to input.
		if player.body then
			local px, py = player:getPosition()
			if love.keyboard.isDown('d', 'right') then
				player:setX(px + player.speed*dt)
			end
			if love.keyboard.isDown('a', 'left') then
				player:setX(px - player.speed*dt)
			end

			if player:enter('danger') then
				player:setPosition(LEVELS[currentLevelIndex].playerStartX, LEVELS[currentLevelIndex].playerStartY)

			elseif player:enter('end') then
				print("Finished level "..currentLevelIndex)

				_G.currentLevelIndex = currentLevelIndex + 1
				if currentLevelIndex > #LEVELS then
					_G.currentLevelIndex = 1 -- Start over.
				end

				startLevel(LEVELS[currentLevelIndex])
			end
		end

		-- Then make collisions happen.
		world:update(dt)

		-- Finally, update the camera now that the player's position won't change anymore.
		local px, py = player:getPosition()
		cam:lookAt(px, love.graphics.getHeight()/2)
	end
end 

function love.draw()
	cam:attach()
	world:draw()
	cam:detach()
end
